package de.unimarburg.swt.emf.metric;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.emf.common.util.TreeIterator;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.util.EcoreUtil;

import simplifiedclassmodel.Attribute;
import simplifiedclassmodel.ScmClass;
import simplifiedclassmodel.SimplifiedClassModel;
import de.unimarburg.swt.emf.metrics.interfaces.ICalculateClass;

public class NEAC implements ICalculateClass {

	private List<EObject> context;

	@Override
	public void setContext(List<EObject> context) {
		this.context=context;
	}

	@Override
	public double calculate() {
		ScmClass cl = (ScmClass) context.get(0);
		return getNumberOfEqualAttributesWithOtherClasses(cl);
	}

	private double getNumberOfEqualAttributesWithOtherClasses(ScmClass cl) {
		double result = 0.0;
		List<ScmClass> otherClasses = getOtherClasses(cl);
		if (otherClasses.isEmpty()) return result;
		Double[] maxEqualAttributes = new Double[otherClasses.size()];
		for (int i=0; i < maxEqualAttributes.length; i++) maxEqualAttributes[i] = 0.0;
		for (Attribute attribute : cl.getOwnedAttribute()) {
			for (int i=0; i < otherClasses.size(); i++) {
				if (containsEqualAttribute(otherClasses.get(i), attribute)) {
					maxEqualAttributes[i]++;
				}
			}		
		}
		result = max(maxEqualAttributes);
		return result;
	}
	
	private double max(Double[] doubleArray) {
		Double result = 0.0;
		for (int i=0; i < doubleArray.length; i++) {
			if (doubleArray[i] > result) {
				result = doubleArray[i];
			}
		}
		return result;
	}
	
	private boolean containsEqualAttribute(ScmClass cl, Attribute attr) {
		boolean contains = false;
		for (Attribute att : cl.getOwnedAttribute()) {
			if (haveEqualNames(attr, att) 
				&& haveEqualTypes(attr, att)
				&& haveEqualVisibility(attr, att)) {
					contains = true;
					break;
				}
		}
		return contains;
	}
	
	private boolean haveEqualVisibility(Attribute attr, Attribute att) {
		return attr.getVisibility().equals(att.getVisibility());
	}

	private boolean haveEqualTypes(Attribute attr, Attribute att) {
		return ((att.getType() == null) && (attr.getType() == null)) || 
				((att.getType() != null) && (attr.getType() != null)) && (att.getType().equals(attr.getType()));
	}

	private boolean haveEqualNames(Attribute attr, Attribute att) {
		return att.getName().equals(attr.getName());
	}

	private List<ScmClass> getOtherClasses(ScmClass cl) {
		List<ScmClass> otherClasses = new ArrayList<ScmClass>();
		SimplifiedClassModel model = (SimplifiedClassModel) EcoreUtil.getRootContainer(cl);
		TreeIterator<EObject> iter = model.eAllContents();
		while (iter.hasNext()) {
			EObject eObject = iter.next();
			if (eObject instanceof ScmClass && eObject != cl) {
				otherClasses.add((ScmClass) eObject);
			}
		}
		return otherClasses;
	}

}
