package de.unimarburg.swt.emf.metric;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.emf.ecore.EObject;

import simplifiedclassmodel.Attribute;
import simplifiedclassmodel.Generalization;
import simplifiedclassmodel.PackageableElement;
import simplifiedclassmodel.ScmClass;
import simplifiedclassmodel.ScmPackage;
import de.unimarburg.swt.emf.metrics.interfaces.ICalculateClass;

public class TNAP implements ICalculateClass {

	private List<EObject> context;

	@Override
	public void setContext(List<EObject> context) {
		this.context=context;
	}

	@Override
	public double calculate() {
		ScmPackage p = (ScmPackage) context.get(0);
		return getAllAttributes(p).size();
	}

	private List<Attribute> getAllAttributes(ScmPackage p) {
		List<ScmClass> classes = new ArrayList<ScmClass>();
		for (PackageableElement pe : p.getOwnedElement()) {
			if (pe instanceof ScmClass) {
				ScmClass cl = (ScmClass) pe;
				if (! cl.isIsAbstract()) classes.add(cl);
			}
		}
		List<Attribute> attributes = new ArrayList<Attribute>();
		for (ScmClass cl : classes) {
			attributes.addAll(getAllAttributes(cl));
		}		
		return attributes;
	}

	private List<Attribute> getAllAttributes(ScmClass cl) {
		List<Attribute> attributes = new ArrayList<Attribute>();
		attributes.addAll(cl.getOwnedAttribute());
		for (Generalization gen : cl.getGeneralization()) {
			ScmClass parentClass = gen.getGeneral();
			attributes.addAll(getAllAttributes(parentClass));
		}
		List<Attribute> redefinedAttributes = new ArrayList<Attribute>();
		for (Attribute attr : attributes) {
			if (! attr.getRedefinedAttribute().isEmpty()) {
				redefinedAttributes.addAll(attr.getRedefinedAttribute());
			}
		}
		attributes.removeAll(redefinedAttributes);
//		System.out.println("Class: " + cl);
//		System.out.println("Attributes: " + attributes.size());
//		System.out.println("Redefined Attributes: " + redefinedAttributes.size());
//		System.out.println("-----------------------------------------------------");
		return attributes;
	}

}
