/**
 * <copyright>
 * </copyright>
 *
 * $Id: RefactoringController.javajet,v 1.3 2011/01/21 13:08:06 tarendt Exp $
 */
package de.unimarburg.swt.scm.refactoringsnew.scm.pullupattribute;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.emf.common.util.TreeIterator;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.refactor.common.core.EmfRefactoring;
import org.eclipse.emf.refactor.common.core.IController;
import org.eclipse.emf.refactor.common.core.IDataManagement;
import org.eclipse.emf.refactor.runtime.ltk.LtkEmfRefactoringProcessorAdapter;
import org.eclipse.ltk.core.refactoring.RefactoringStatus;
import org.eclipse.ltk.core.refactoring.participants.RefactoringProcessor;

import simplifiedclassmodel.Attribute;
import simplifiedclassmodel.Generalization;
import simplifiedclassmodel.ScmClass;

public final class RefactoringController implements IController{

	/**
	 * EmfRefactoring supported by the controller.
	 * @generated
	 */
	private EmfRefactoring parent;
	
	/**
	 * DataManagement object of the model refactoring.
	 * @generated
	 */
	private RefactoringDataManagement dataManagement = 
									new RefactoringDataManagement();
	
	/**
	 * Invocation context of the model refactoring.
	 * @generated
	 */	
	private List<EObject> selection = new ArrayList<EObject>();
	
	/**
	 * Ltk RefactoringProcessor of the model refactoring.
	 * @generated
	 */
	private InternalRefactoringProcessor refactoringProcessor = null;
	
	/**
	 * Gets the EmfRefactoring supported by the controller.
	 * @return EmfRefactoring supported by the controller.
	 * @see org.eclipse.emf.refactor.common.core.IController#getParent()
	 * @generated
	 */
	@Override
	public EmfRefactoring getParent() {
		return this.parent;
	}
	
	/**
	 * Sets the EmfRefactoring supported by the controller.
	 * @param emfRefactoring EmfRefactoring supported by the controller.
	 * @see org.eclipse.emf.refactor.common.core.IController#
	 * setParent(org.eclipse.emf.refactor.common.core.EmfRefactoring)
	 * @generated
	 */
	@Override
	public void setParent(EmfRefactoring emfRefactoring) {
		this.parent = emfRefactoring;
	}
	
	/**
	 * Returns the DataManagement object of the model refactoring.
	 * @return DataManagement object of the model refactoring.
	 * @see org.eclipse.emf.refactor.common.core.IController#
	 * getDataManagementObject()
	 * @generated
	 */
	@Override
	public IDataManagement getDataManagementObject() {
		return this.dataManagement;
	}

	/**
	 * Returns the ltk RefactoringProcessor of the model refactoring.
	 * @return Ltk RefactoringProcessor of the model refactoring.
	 * @see org.eclipse.emf.refactor.common.core.IController#
	 * getLtkRefactoringProcessor()
	 * @generated
	 */
	@Override
	public RefactoringProcessor getLtkRefactoringProcessor() {
		return this.refactoringProcessor;
	}
	
	/**
	 * Sets the selected EObject (invocation context of the model refactoring).
	 * @param selection Invocation context of the model refactoring.
	 * @see org.eclipse.emf.refactor.common.core.IController#
	 * setSelection(java.util.List)
	 * @generated
	 */
	@Override
	public void setSelection(List<EObject> selection) {
		this.selection = selection;
		this.refactoringProcessor = 
				new InternalRefactoringProcessor(this.selection);
	}	
	
	/**
	 * Returns a Runnable object that executes the model refactoring.
	 * @return Runnable object that executes the model refactoring.
	 * @generated
	 */
	private Runnable applyRefactoring() {
		return new Runnable() {				
			/**
			 * @see java.lang.Runnable#run()
			 * @generated
			 */
			@Override
			public void run() {
				simplifiedclassmodel.Attribute selectedEObject = 
					(Attribute) dataManagement.getInPortByName(dataManagement.SELECTEDEOBJECT).getValue();
				String superclassname =
					(String) dataManagement.getInPortByName("superclassname").getValue();
				ScmClass owningClass = selectedEObject.getOwningClass();
				ScmClass superclass = getSuperclassWithName(owningClass, superclassname);
				owningClass.getOwnedAttribute().remove(selectedEObject);
				superclass.getOwnedAttribute().add(selectedEObject);
				selectedEObject.setOwningClass(superclass);
				List<ScmClass> subclasses = getAllSubclasses(superclass);
				for (ScmClass subclass : subclasses) {
					removeEqualAttributeFromClass(subclass, selectedEObject);
				}
			}
		};
	}
	
	private boolean containsEqualAttribute(ScmClass cl, Attribute attr) {
		boolean contains = false;
		for (Attribute att : cl.getOwnedAttribute()) {
			if (haveEqualNames(attr, att) 
				&& haveEqualTypes(attr, att)
				&& haveEqualVisibility(attr, att)) {
					contains = true;
					break;
				}
		}
		return contains;
	}
	
	private void removeEqualAttributeFromClass(ScmClass scmClass,
			Attribute attr) {
		Attribute equalAttribute = null;
		for (Attribute att : scmClass.getOwnedAttribute()) {
			if (haveEqualNames(attr, att) 
				&& haveEqualTypes(attr, att)
				&& haveEqualVisibility(attr, att)) {
					equalAttribute = att;
					break;
				}
		}
		scmClass.getOwnedAttribute().remove(equalAttribute);
	}
	
	private boolean haveEqualVisibility(Attribute attr, Attribute att) {
		return attr.getVisibility().equals(att.getVisibility());
	}

	private boolean haveEqualTypes(Attribute attr, Attribute att) {
		return ((att.getType() == null) && (attr.getType() == null)) || 
				((att.getType() != null) && (attr.getType() != null)) && (att.getType().equals(attr.getType()));
	}

	private boolean haveEqualNames(Attribute attr, Attribute att) {
		return att.getName().equals(attr.getName());
	}

	private List<ScmClass> getAllSubclasses(ScmClass superclass) {
		List<ScmClass> subclasses = new ArrayList<ScmClass>();
		TreeIterator<EObject> iter = superclass.eResource().getAllContents();
		while (iter.hasNext()) {
			EObject eObject = iter.next();
			if (eObject instanceof ScmClass) {
				ScmClass scmClass = (ScmClass) eObject;
				for (Generalization gen : scmClass.getGeneralization()) {
					if (gen.getGeneral() == superclass) {
						subclasses.add(scmClass);
					}
				}
			}			
		}
		return subclasses;
	}

	private ScmClass getSuperclassWithName(ScmClass scmClass,
			String superclassname) {
		for (Generalization gen : scmClass.getGeneralization()) {
			if (gen.getGeneral().getName().equals(superclassname)) {
				return gen.getGeneral() ;
			}
		}
		return null;
	}

	/**
	 * Internal class for providing an instance of a LTK RefactoringProcessor 
	 * used for EMF model refactorings.	 
	 * @generated
	 */
	public final class InternalRefactoringProcessor extends 
									LtkEmfRefactoringProcessorAdapter {

		/**
		 * Constructor using the invocation context of the model refactoring.
		 * @param selection Invocation context of the model refactoring.
		 * @generated
		 */
		private InternalRefactoringProcessor(List<EObject> selection){
				super(getParent(), selection, applyRefactoring());				
		}
			
		/**
		 * @see org.eclipse.ltk.core.refactoring.participants.RefactoringProcessor#
	 	 * checkInitialConditions(org.eclipse.core.runtime.IProgressMonitor)
		 * @generated
		 */	
		@Override
		public RefactoringStatus checkInitialConditions(){
				RefactoringStatus result = new RefactoringStatus();
				simplifiedclassmodel.Attribute selectedEObject = 
					(simplifiedclassmodel.Attribute) dataManagement.
							getInPortByName(dataManagement.SELECTEDEOBJECT).getValue();
				// TODO: implement initial check
				return result;
		}
		
		/**
		 * @see org.eclipse.ltk.core.refactoring.participants.RefactoringProcessor#
	     * checkFinalConditions(org.eclipse.core.runtime.IProgressMonitor, 
	     * org.eclipse.ltk.core.refactoring.participants.CheckConditionsContext)
		 * @generated
		 */	
		@Override
		public RefactoringStatus checkFinalConditions(){
				RefactoringStatus result = new RefactoringStatus();
				Attribute selectedEObject = 
					(Attribute) dataManagement.getInPortByName(dataManagement.SELECTEDEOBJECT).getValue();
				String superclassname =
					(String) dataManagement.getInPortByName("superclassname").getValue();
				// check: each sibling class must have an equal attribute
				ScmClass owningClass = selectedEObject.getOwningClass();
				ScmClass superclass = getSuperclassWithName(owningClass, superclassname);
				List<ScmClass> subclasses = getAllSubclasses(superclass);
				for (ScmClass subclass : subclasses) {
					if (! containsEqualAttribute(subclass, selectedEObject)) {
						result.addFatalError("There is a sibling class which does not own an attribute equal to the selected one!");
					}
				}				
				return result;
		}
		
	}

}