package de.unimarburg.swt.uml.metrics.umlpackage;

import java.io.IOException;
import java.net.URL;
import java.util.Collections;
import java.util.List;
import java.util.Stack;
import org.eclipse.core.runtime.FileLocator;
import org.eclipse.core.runtime.Path;
import org.eclipse.emf.ecore.EObject;
import de.unimarburg.swt.emf.metrics.calculator.henshin.HenshinUtils;
import de.unimarburg.swt.emf.metrics.interfaces.ICompositeCalculateClass;
import de.unimarburg.swt.emf.metrics.model.HenshinTransformationsPack;
import de.unimarburg.swt.emf.metrics.operations.*;


public final class NAVCP implements ICompositeCalculateClass{

	private final HenshinTransformationsPack[] transformations = {
			new HenshinTransformationsPack("", getFullPath("transformations/ncp.henshin"), ""), 
			new HenshinTransformationsPack("", getFullPath("transformations/nap.henshin"), "")};
	private final IOperation[] operations = {new DivisionOperation()};
	private List<EObject> context;
	
	
	@Override
	public void setContext(List<EObject> context){
		this.context=context;
	}
	
	@Override
	public double calculate() {
		Stack<Double> valStack = new Stack<Double>();
		for(HenshinTransformationsPack transformation : transformations){
			valStack.push(value(transformation));
		}
		Stack<IOperation> opStack = new Stack<IOperation>();
		for(IOperation operation : operations){
			opStack.push(operation);
		}
		return result(opStack.pop(), valStack.pop(), valStack.pop());
	}
		
	@Override
	public String getCalculateOperation(){
		return "result(opStack.pop(), valStack.pop(), valStack.pop())";
	}
	
	@Override
	public HenshinTransformationsPack[] getTransformationPacks() {
		return transformations;
	}

	@Override
	public IOperation[] getOperations() {
		return operations;
	}
		
	private String getFullPath(String transformationPath){
		URL url = FileLocator.find(de.unimarburg.swt.uml.metrics.Activator.getDefault().getBundle(), new Path(transformationPath), Collections.EMPTY_MAP);
		URL fileUrl = null;
		try {
		fileUrl = FileLocator.toFileURL(url);
		}
		catch (IOException e) {
		e.printStackTrace();
		}
		return 	fileUrl.getPath();
	}	
		
	private double result(IOperation operation, double a, double b){
		return operation.calculate(a, b);
	}
	
	private double value(HenshinTransformationsPack transformations){
		return HenshinUtils.run(transformations, this.context);
	}
}
